// cristinel.ababei@marquette.edu; copyleft 2023
// this implements communication with a Wii NunChuck controller;
// it reads periodically all the NunChuck data and displays it
// on an LCD display;

#ifndef _NUNCHUCK_H_
#define _NUNCHUCK_H_

#ifdef __cplusplus
 extern "C" {
#endif

#include "main.h"

#define BUFSIZE 64
#define DEFAULT_FONT FONT_6X8

////////////////////////////////////////////////////////////////////////////////
//
// NUNCHUCK
//
////////////////////////////////////////////////////////////////////////////////

class NUNCHUCK {
	private:
    I2C_HandleTypeDef *_hi2c; // pointer to the i2c peripheral;
    uint8_t _address_slave1; // should be 0xA4
    uint8_t _address_slave2; // should be 0xA5
    
    // buffers to store data we want to transmit or that we receive;
    // I create them of fixed size, BUFSIZE=64 entries, to keep things simple;
    uint8_t _master_transmit_buffer[BUFSIZE]; 
    uint8_t _slave_receive_buffer[BUFSIZE];
    // a small sketch buffer for printing to LCD;
    char _text_buffer[8];

	public:
    uint16_t _joy_x_axis;
    uint16_t _joy_y_axis;
    uint16_t _accel_x_axis;
    uint16_t _accel_y_axis;
    uint16_t _accel_z_axis;
    uint16_t _z_button;
    uint16_t _c_button;

	public:
		NUNCHUCK(I2C_HandleTypeDef *hi2c, 
      uint8_t address_slave1, uint8_t address_slave2) : _hi2c(hi2c), 
      _address_slave1(address_slave1), _address_slave2(address_slave2) {
      _joy_x_axis = 0;
      _joy_y_axis = 0;
      _accel_x_axis = 0;
      _accel_y_axis = 0;
      _accel_z_axis = 0;
      _z_button = 0;
      _c_button = 0;
    }
		~NUNCHUCK() {}

		void set_address_slave1(uint8_t address_slave1) { _address_slave1 = address_slave1; }
    void set_address_slave2(uint8_t address_slave2) { _address_slave2 = address_slave2; }
    void set_i2c_handler(I2C_HandleTypeDef *hi2c) { _hi2c = hi2c; }
       
		uint8_t address_slave1() { return _address_slave1; }
    uint8_t address_slave2() { return _address_slave2; }
    
		uint16_t joy_x_axis() { return _joy_x_axis; }
    uint16_t joy_y_axis() { return _joy_y_axis; }
    uint16_t accel_x_axis() { return _accel_x_axis; }
    uint16_t accel_y_axis() { return _accel_y_axis; }
    uint16_t accel_z_axis() { return _accel_z_axis; }
    uint16_t z_button() { return _z_button; }
    uint16_t c_button() { return _c_button; }

    // clear the receive buffer; get ready for a new receive;
    void clear_receive_buffer() {
      for (int i = 0; i < BUFSIZE; i++)
        _slave_receive_buffer[i] = 0x00;
    }
    // functions to write and to read from NunChuck;
    HAL_StatusTypeDef Write_To_NunChuck(uint16_t DevAddress, uint8_t *pData, uint16_t len);
    HAL_StatusTypeDef Read_From_NunChuck(uint16_t DevAddress, uint8_t *pData, uint16_t len);
    // utility functions;
    void print_data_initially(void);
    void phase1_init(void);
    void phase2_read(void);
    void translate_data(void);
    void print_data_new_readings(void);
};

#ifdef __cplusplus
}
#endif

#endif // _NUNCHUCK_H_
